package com.Game;

import android.app.Activity;
import android.os.Bundle;
import android.content.res.Resources;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.graphics.Typeface;
import android.graphics.Paint;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.MotionEvent;
import android.widget.TextView;
import android.util.Log;
import java.util.Random;

import com.Game.GameThread;
import com.Game.GamePlayer;
import com.Game.GameEnemy;
import com.Game.GameBullet;
import com.Game.GameMenu;
import com.Game.Vector2D;
import com.Game.Constants;

public class GameView extends SurfaceView implements SurfaceHolder.Callback
{
	private final String LOG = "LOG";
	
	private GameThread 	thread;
	private Context 	mContext;
	private Paint		mTextPaint;
	private Typeface	mFont = Typeface.create(Typeface.SANS_SERIF, Typeface.BOLD);	
	
	//Images
	private Drawable 	mPlayerImg;
	private Drawable 	mEnemyImg;
	private Drawable	mBulletImg;
	
	//Images 2
	private Bitmap		mArrow[];
	private Bitmap		mBG;
	
	//Sprites classes
	private GameEnemy 	mEnemy[];
	private GamePlayer 	mPlayer;
	private GameBullet 	mPlayerBullet[];
	private GameBullet 	mEnemyBullet;
	private GameMenu	mGameMenu[];
	
	//Game Related
	private int 		mLevelID 				= 0;
	private int 		mTextHeightDiff			= 0;
	private static int 	mNextBulletID			= 0;
	private boolean  	bIsScreenTouched		= false;	
	private Vector2D 	mTouchPos 				= new Vector2D();	
	private static int 	mGameState;
	
	//Strings
	private static String TXT_GAME_NAME;
	private static String TXT_PLAY;
	private static String TXT_PAUSE;
	private static String TXT_ABOUT;
	private static String TXT_ABOUT_TEXT;
	private static String TXT_OPTION;
	private static String TXT_SCORE;
	private static String TXT_HEALTH;

	//
	public int[] mEnemyCount = { 7, 9, 11, 13, 15 };
	
	//Methods
	public GameView(Context context)
	{
		super(context);
		
		mContext = context;
		thread = new GameThread(getHolder(), this);
		getHolder().addCallback(this);
		
		mGameState = Constants.STATE_MENU;			//debug, STATE_SPLASHANDINTRO
		
		mTextPaint = new Paint();
		mTextPaint.setTextSize(25);
		mTextPaint.setTypeface(mFont);
		mTextPaint.setAntiAlias(true);
		mTextPaint.setDither(true);
		
		int width = Constants.SCREEN_WIDTH;
		int height = Constants.SCREEN_HEIGHT;
			
		Resources res = mContext.getResources();
		
		//load images
		mPlayerImg 	= res.getDrawable(R.drawable.catapult);
		mEnemyImg 	= res.getDrawable(R.drawable.paratrooper);
		mBulletImg	= res.getDrawable(R.drawable.bullet);
		
		//////////////////////////////////////////////////////////////
		//player
		mPlayer = new GamePlayer(mPlayerImg, new Vector2D(width / 2, height - mPlayerImg.getIntrinsicHeight() / 2), Constants.MAX_PLAYER_LIFE, width, height);
		
		//////////////////////////////////////////////////////////////
		//player's bullet
		mPlayerBullet = new GameBullet[Constants.MAX_BULLETS];
		for(int i = 0; i < Constants.MAX_BULLETS; i++)
		{
			mPlayerBullet[i] = new GameBullet(mBulletImg, new Vector2D(mPlayer.mPosition), width, height);
			mPlayerBullet[i].bIsAlive = true;
		}
		
		//////////////////////////////////////////////////////////////
		//enemy
		int enemySpawnMaxX = width - mEnemyImg.getIntrinsicWidth();
		int enemySpawnY = -mEnemyImg.getIntrinsicHeight();
		mEnemy = new GameEnemy[Constants.MAX_ENEMY];
		for(int i = 0; i < Constants.MAX_ENEMY; i++)
		{
			mEnemy[i] = new GameEnemy(mEnemyImg, new Vector2D(getRandom(enemySpawnMaxX), enemySpawnY), getRandom(width / 4, height / 2), width, height);
			
			if(i < mEnemyCount[mLevelID])
				mEnemy[i].bIsAlive = true;
		}
		
		mEnemy[0].bCanJump = true;
		
		//////////////////////////////////////////////////////////////
		//menu		
		int posX = width - (height / Constants.HEIGHT_DIVIDER);
		int offsetX = width / Constants.HEIGHT_DIVIDER;		
		mGameMenu = new GameMenu[Constants.MENU_ITEM_COUNT];
		for(int i = 0; i < Constants.MENU_ITEM_COUNT; i++)
			mGameMenu[i] = new GameMenu(mEnemyImg, new Vector2D(((posX * i) / Constants.MENU_ITEM_COUNT) + offsetX , 0), Constants.HEIGHT_DIVIDER, width, height);
		
		//////////////////////////////////////////////////////////////
		//nullify images
		mPlayerImg 	= null;
		mEnemyImg 	= null;
		mBulletImg 	= null;	
	}
	
	public GameThread getThread() { return thread; }
	
	private void reset(int level)
	{	
		int screenWidth = mPlayer.mScreenWidth;
		int screenHeight = mPlayer.mScreenHeight;
		
		//player
		mPlayer.mHealth = mPlayer.HEALTH_MAX;
		mPlayer.SetPosition(screenWidth / 2 - mPlayer.mDimention.x / 2, screenHeight - mPlayer.mDimention.y);
		
		//player's bullet
		for(int i = 0; i < Constants.MAX_BULLETS; i++)
		{
			mPlayerBullet[i].mPosition = mPlayer.mPosition;
			mPlayerBullet[i].bIsAlive = false;
		}
			
		//enemy
		for(int i = 0; i < mEnemyCount[level]; i++)
		{
			mEnemy[i].bIsAlive	= true;
			mEnemy[i].bCanJump	= false;
			
			int enemySpawnMaxX = screenWidth - mEnemy[i].mDimentionHalf.x;
			int enemySpawnY = - mEnemy[i].mDimention.y;
			mEnemy[i].SetPosition(getRandom(enemySpawnMaxX), enemySpawnY);
			mEnemy[i].mNextSpawnY = getRandom(screenWidth / 4, screenWidth / 2);
		}
		
		mEnemy[0].bCanJump = true;
	
		mGameState = Constants.STATE_GAME;
	}
	
	@Override
	public void surfaceCreated(SurfaceHolder holder)
	{		
		setFocusable(true);
		
		thread.setRunning(true);
		thread.start();
	}
	
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height)
	{		
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder)
	{
		boolean retry = true;
		thread.setRunning(false);
		while(retry)
		{
			try
			{
				thread.join();
				retry = false;
			}
			catch(InterruptedException e)
			{
			}
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	////TOUCH
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	
	@Override
	public boolean onTouchEvent(MotionEvent event)
	{
		synchronized(thread.getSurfaceHolder())
		{
			switch(event.getAction())
			{	
				case MotionEvent.ACTION_DOWN:
					onMenuTouch(event);
					onTouchEventDown(event);
				break;
				
				case MotionEvent.ACTION_UP:
					switch(mGameState)
					{
						case Constants.STATE_GAME:						
							bIsScreenTouched = false;
						break;
					}
				break;
				
				case MotionEvent.ACTION_MOVE:
				break;
			}
		}
		
		return super.onTouchEvent(event);
	}
	
	//Add events to occur when the screen is touched down
	private void onTouchEventDown(MotionEvent event)
	{
		switch(mGameState)
		{
			case Constants.STATE_MENU:
				mGameState = Constants.STATE_GAME;
			break;
				
			case Constants.STATE_GAME:
				mTouchPos = new Vector2D((int)event.getX(), (int)event.getY());
				bIsScreenTouched = true;
			break;
			
			case Constants.STATE_PAUSE:
				//
			break;
			
			case Constants.STATE_LEVELEND:
				//
			break;
			
			case Constants.STATE_QUIT:
				//thread.stop();
			break;
		}
	}
	
	private void onMenuTouch(MotionEvent event)
	{		
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	/////DRAW
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	
	@Override
	public void onDraw(Canvas canvas)
	{
		//draw BG
		canvas.drawColor(Color.BLACK);		//TODO: replace with some image
		
		//draw game sprites
		canvas.save();
		{
			switch (mGameState)
			{
				case Constants.STATE_MENU:
					onDrawMenu(canvas);
				break;
					
				case Constants.STATE_GAME:
					onDrawGame(canvas);
				break;	
			}	
			
			onDrawText(canvas);
		}
		canvas.restore();
	}
	
	private void onDrawMenu(Canvas canvas)
	{
		
		for(int i = 0; i < Constants.MENU_ITEM_COUNT; i++)
			mGameMenu[i].DrawSprite(canvas);		
	}
	
	private void onDrawGame(Canvas canvas)
	{	
		mPlayer.DrawSprite(canvas);
		
		for(int i = 0; i < Constants.MAX_BULLETS; i++)
			mPlayerBullet[i].DrawSprite(canvas);
			
		for(int i = 0; i < mEnemyCount[mLevelID]; i++)
			mEnemy[i].DrawSprite(canvas);
	}
	
	private void onDrawText(Canvas canvas)
	{
		switch(mGameState)
		{
			case Constants.STATE_MENU:
				//title
				mTextPaint.setTextSize(21);
				mTextPaint.setColor(Color.RED);
				mTextPaint.setTextAlign(Paint.Align.CENTER);
				//canvas.drawText(TXT_GAME_NAME, mScreenWidthHalf, mScreenHeightQuater - (mTextHeightDiff * 2), mTextPaint);				
			break;
			
			case Constants.STATE_GAME:
				//score
				mTextPaint.setColor(Color.WHITE);
				mTextPaint.setTextAlign(Paint.Align.LEFT);
				//canvas.drawText(TXT_SCORE + mScore, 0, mTextHeightDiff, mTextPaint);
				
				//health
				mTextPaint.setTextAlign(Paint.Align.RIGHT);
				//canvas.drawText(TXT_HEALTH + mPlayerHealth, mScreenWidth, mTextHeightDiff, mTextPaint);
			break;
			
			case Constants.STATE_PAUSE:
				//
			break;
			
			case Constants.STATE_LEVELEND:
				//
			break;
			
			case Constants.STATE_OPTION:
				//
			break;
			
			case Constants.STATE_ABOUT:
				mTextPaint.setTextSize(18);
				mTextPaint.setColor(Color.WHITE);
				mTextPaint.setTextAlign(Paint.Align.LEFT);
				
				canvas.drawText(TXT_ABOUT_TEXT, 0, mTextHeightDiff, mTextPaint);
			break;
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	////Gameplay
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	
	private void onLevelEnd()
	{
		//Display score, bonus, level info
		reset(mLevelID);
	}
	
	private void windEffect(int velocity)
	{
		//TODO
	}
	
	private void gameProgress()
	{
	}
	
	private void GameLoopBullet()
	{
		for(int i = 0; i < Constants.MAX_BULLETS; i++)
		{
			mPlayerBullet[i].GameLoop(Constants.MOVE_SPEED, mPlayer, mPlayer.bBulletFired);
			//mPlayer.bBulletFired = mPlayerBullet[i].onBulletFired(mPlayer.bBulletFired);
		}
	}
	
	private void GameLoopEnemy()
	{
		for(int i = 0; i < mEnemyCount[mLevelID]; i++)
		{
			mEnemy[i].GameLoop(Constants.MOVE_SPEED);
			
			for(int j = 0; j < Constants.MAX_BULLETS; j++)
				if(mEnemy[i].IsCollidingWith(mPlayerBullet[j]))
					mEnemy[i].onColliding(mPlayerBullet[j]);
			
			if(mEnemy[i].IsCollidingWith(mPlayer))
					mEnemy[i].onColliding(mPlayer, Constants.DAMAGE_HP);
					
			if(mEnemy[i].mPosition.y > mEnemy[i].mNextSpawnY && i < mEnemyCount[mLevelID] - 1)
				mEnemy[i + 1].bCanJump = true;
		}
	}
	
	public void gameLoop(Canvas canvas)
	{
		onDraw(canvas);
		gameProgress();
		
		switch(mGameState)
		{
			case Constants.STATE_MENU:
				for(int i = 0; i < Constants.MENU_ITEM_COUNT; i++)
					mGameMenu[i].GameLoop(i, Constants.MOVE_SPEED);
			break;
			
			case Constants.STATE_GAME:
				mPlayer.GameLoop(mTouchPos, Constants.MOVE_SPEED);
				GameLoopBullet();
				GameLoopEnemy();
			break;
		}
		
		debug();
	}
	
	private void debug()
	{	
	}
	
	////////////////////////////////////////////////////////////////////////////////
	
	private int getRandom(int max)
	{
		Random random = new Random();		
		return Math.abs(random.nextInt() % max);
	}
	
	private int getRandom(int min, int max)
	{
		int num = -1;
		Random random = new Random();
		
		while(num < min)
			num = Math.abs(random.nextInt() % max);
			
		return num;
	}
	
	////////////////////////////////////////////////////////////////////////////////
}